/**
 * Copyright (c) 2002-2015 "Neo Technology,"
 * Network Engine for Objects in Lund AB [http://neotechnology.com]
 *
 * This file is part of Neo4j.
 *
 * Neo4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.neo4j.ndp.messaging.v1;

import java.io.IOException;
import java.nio.channels.ReadableByteChannel;
import java.nio.channels.WritableByteChannel;

import org.neo4j.ndp.messaging.v1.message.Message;

public interface MessageFormat
{
    Writer newWriter();

    Reader newReader();

    int version();

    interface Writer extends MessageHandler<IOException>
    {
        Writer write( Message msg ) throws IOException;

        Writer reset( WritableByteChannel channel );

        void flush() throws IOException;
    }

    interface Reader
    {
        /** Return true if there is another message in the underlying buffer */
        boolean hasNext() throws IOException;

        <E extends Exception> void read( MessageHandler<E> consumer ) throws IOException, E;

        Reader reset( ReadableByteChannel channel );
    }
}
